function [c]=TMS2(ima,k,mascara)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%    TMS: Trimmed Means Segmentation
%
%    Author: Jose Vicente Manjon Herrera         
%     Email: jmanjon@fis.upv.es
%      Date: 13-04-2005
%
%   Usage:[c]=TMS(ima,k,res)
%
%   Input: 
%
%         ima: Image or volume to be segmented
%           k: Number of classes
%         res: Voxel resolution
%
%  Output:
%       
%          c: Vector of class means  
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

alfa=0.05;% 5% de los datos de menor gradiente

vima=cgradientmodule(ima);
ind=find(ima);
m=std(vima(ind));

if(nargin==2)
 [mascara,ce]=rkmeans(ima,(vima<=2*m),k);
end
mascara=cTruncado(mascara);

for i=1:k,
    
  ind=find(mascara==i);
  p=ima(ind);  
  v=vima(ind);
    
  h=histograma(v);
      
  t2=sum(h)*alfa;
  acu=0;
  n=1;
  while(1) 
    acu=acu+h(n);
    if(acu>t2) break;end;
    n=n+1;
  end
  ind=find(v<n);
  p=p(ind);

  ca(i)=FastLTSloc(p); 
  
end

c=sort(ca);


function[h]=histogram(datos)

datos=datos(:);
tam=length(datos);
m=ceil(max(datos))+1;
h=zeros(m,1);

for i=1:tam,
    f=floor(datos(i));    
    if(f>0 & datos(i)<=m)        
        a2=datos(i)-f;
        a1=1-a2;
        h(f)  =h(f)  + a1;      
        h(f+1)=h(f+1)+ a2;                          
    end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Fast LTS Estimator
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function loc=FastLTSloc(X)

% If X is a row vector make it a column vector
if size(X,1)==1
   X=X';
end

% n is the length of the data set
n=size(X,1);

% For a single data point there is no need to proceed
if n==1
   loc=X;
else
   % Calculate the location parameter for every column of X
   for i=1:size(X,2);
            
      histo=histogram(X(:,i));
      
      % P. 169: the length of the "half" of the data points
      h=floor(n/2)+1;
      
      % P. 171: optimize eq. 2.8
      
      a=find(histo>0);
      minimo=a(1);
      maximo=a(length(a));
      
      SQmin=Inf;            
      for j=minimo:maximo,         
         
         jj=j;
         
         for k=0:histo(jj) 
         
         na=0;
         acu=0; 
         
         while(jj<=maximo)             
             if((na+histo(jj))>h)                
               acu=acu+(h-na)*jj;
               na=h;
               break;
             else
               if(jj==j)
                 na=na+(histo(jj)-k);
                 acu=acu+(histo(jj)-k)*jj;
               else
                 na=na+histo(jj);
                 acu=acu+histo(jj)*jj;
               end
             end
             jj=jj+1;
         end         
         if(na<h) break; end;             
         media=acu/h;
         
         na=0;
         acu=0;
         jj=j;
         while(jj<=maximo)             
             if((na+histo(jj))>h)                
               acu=acu+(h-na)*(jj-media)^2;
               na=h;
               break;
             else
               if(jj==j)
                 na=na+(histo(jj)-k);
                 acu=acu+(histo(jj)-k)*(jj-media)^2;
               else
                 na=na+histo(jj);
                 acu=acu+histo(jj)*(jj-media)^2;
               end
             end
             jj=jj+1;
         end
         SQ=sqrt(acu/(h-1));
                  
         if SQ<SQmin
            SQmin=SQ;
            loc=media;
         end         
         
         end
      end
   end
end


    
function [mask,bestmu] = rkmeans(imag,mask,k)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   robust kmeans clustering algorithm.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%55

% % variable definition
% 
% ima=round(imag);
% s=size(ima);
% 	
% % histogram calculation
% 
% ind=find(ima>0);
% Min=floor(min(ima(ind)));
% Max=ceil(max(ima(ind)));
% bins=round(Max);
% histo=zeros(bins,1);
% t=length(ind);
% for n=1:t,
%   if(mask(ind(n))>0)  
%     histo(ima(ind(n)))=  histo(ima(ind(n)))+1;
%   end
% end;   

histo=histogram(imag);

% incremental initialization (2)
ind=find(histo);
mu(1)=sum(histo(ind).*ind)/sum(histo(ind));
mu(2)=mu(1)+1;
mu(1)=mu(1)-1;
[mu,v] = findmeans(histo,mu);
m(1)=mu(1)+1;
m(2)=mu(1)-1;
m(3)=mu(2);  
[bestmu1,v1,error1] = findmeans(histo,m);
m(1)=mu(2)+1;
m(2)=mu(2)-1;
m(3)=mu(1);
[bestmu2,v2,error2] = findmeans(histo,m);

if(error1<error2) bestmu=bestmu1;
else bestmu=bestmu2;
end

% calculate mask
mask=cCreatemask(imag,3,bestmu);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [mu,v,error] = findmeans(h,mu)

lh=length(h);
k=length(mu);
hc=zeros(lh,1);

% start process

while(true)
  
  oldmu=mu;
  % current classification  
 
  for i=1:lh
      c=abs(i-mu);
      cc=find(c==min(c));
      hc(i)=cc(1);
  end
  
  %recalculation of means  
  
  for i=1:k, 
      a=find(hc==i);
      if(sum(h(a))==0) continue; end;      
      mu(i)=sum(a.*h(a))/sum(h(a));
  end
  
  if(mu==oldmu) break;end;
  
end

error=0;
for i=1:k
      a=find(hc==i);     
      if(length(a)==0) continue;
      else error = error + sum(abs(a-mu(i)).*h(a)); 
      end      
end

v=zeros(1,k);
for i=1:k, 
      a=find(hc==i);
      if(sum(h(a))==0) continue; end;      
      v(i)=sum((a-mu(k)).*(a-mu(k)).*h(a))/(sum(h(a))-1);
end

mu=sort(mu);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
