function segment(filename)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Segment a  MRI T1w volume in CSF,GM and WM tissues
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
warning off;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% load data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if(nargin==0)
  filename=spm_select;
end

disp('Loading data ...');

[path,name,ext]=fileparts(filename);
path=[path,'\'];
% cd ..
%resultspath=[pwd,'\results\',name,'\'];
% cd modules
fname=[name,ext];
%mkdir(resultspath)

V=spm_vol(filename);
ima=spm_read_vols(V);
ima=double(ima);

%************************************************
% denoising
%************************************************

disp('Denoising ...');

tic
h=RicianSTD(ima); 
disp(['Estimated noise = ',num2str(h)]);
%fima=cM_VNLM3D(ima,3,1,h,1);
fima=MBONLM3D(ima,3,1,h,1);

% save residuals
V.fname=[path,'res_',fname];
spm_write_vol(V,ima-fima);

% save data
V.fname=[path,'f_',fname];
spm_write_vol(V,fima);

tdenoise=toc;

%************************************************
% Brain Extraction and biascorrection 
%************************************************

disp('Brain Extraction and Bias Correction ...');

tic
[bima,label]=exbrain(V.fname);

% save data
brain=bima.*double(label>0);

V.fname=[path,'e_',fname];
spm_write_vol(V,brain);

V.fname=[path,'mask_',fname];
spm_write_vol(V,double(label>0));

tbias=toc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% clasification de tejidos
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5

disp('Segmenting ...');
tic
label=round(label);
c=TMS(brain,3,label)
mascara=cCreateMaskSoft(brain,3,c);

V.dt(1)=16;
V.pinfo(1)=1;
V.fname=[path,'csf_',fname];
spm_write_vol(V,double(squeeze(mascara(:,:,:,1))));
V.fname=[path,'gm_',fname];
spm_write_vol(V,double(squeeze(mascara(:,:,:,2))));
V.fname=[path,'wm_',fname];
spm_write_vol(V,double(squeeze(mascara(:,:,:,3))));
V.dt(1)=8;
V.fname=[path,'crisp_',fname];
spm_write_vol(V,double(squeeze(mascara(:,:,:,4))));

mascara=refine(brain,mascara,c);

V.dt(1)=16;
V.pinfo(1)=1;
V.fname=[path,'new_csf_',fname];
spm_write_vol(V,double(squeeze(mascara(:,:,:,1))));
V.fname=[path,'new_gm_',fname];
spm_write_vol(V,double(squeeze(mascara(:,:,:,2))));
V.fname=[path,'new_wm_',fname];
spm_write_vol(V,double(squeeze(mascara(:,:,:,3))));
V.dt(1)=8;
V.fname=[path,'new_crisp_',fname];
spm_write_vol(V,double(squeeze(mascara(:,:,:,4))));


tseg=toc;
 
disp('Done!');

% tissue volumes

csf=mascara(:,:,:,1);
csfvol=sum(csf(:))/1000 
gm=mascara(:,:,:,2);
gmvol=sum(gm(:))/1000 
wm=mascara(:,:,:,3);
wmvol=sum(wm(:))/1000 

tiv=(gmvol+wmvol+csfvol) % Total intracraneal volume
ncsf=csfvol/tiv % normalized csf volume
ngm=gmvol/tiv % normalized gray matter volume
nwm=wmvol/tiv % normalized white matter volume
nbv=(gmvol+wmvol)/tiv % normalized brain volume 

fn=[path,'\volumes.txt'];
if (~exist(fn, 'file'))
    fid = fopen(fn,'a+');
    fprintf(fid,'Filename\t   CsfVol(mm3)\t     GmVol(mm3)\t     WmVol(mm3)\t     TIV(mm3)\t     Ncsf(%%)\t     Ngm(%%)\t     Nwm(%%)\t     Nbv(%%)\r\n');
    fclose(fid);
end
fid = fopen(fn,'a+');
fprintf(fid,'%s\t %1.2f\t %1.2f\t %1.2f\t %1.2f\t %1.2f\t %1.2f\t %1.2f\t %1.2f\r\n', fname, csfvol, gmvol, wmvol, tiv, ncsf, ngm, nwm, nbv);
fclose(fid);
