% This function reorients coronal or sagittal volume into axial.
% In: 
%   - arg: To know if volume is in axial, coronal or sagittal orientation.
%   - file: Absolut path to the file to reorient.
%
% Out:
%   - completePath: Absolut path to the reoriented file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [completePath] = reorientation(arg, file, pathSave)


  V = spm_vol(file);
  
  volOriginal=spm_read_vols(V);
  
  res(1)=abs(V.mat(1,1));
  res(2)=abs(V.mat(2,2));
  res(3)=abs(V.mat(3,3));
  
  origin(1)=V.mat(1,4);
  origin(2)=V.mat(2,4);
  origin(3)=V.mat(3,4);
  
  stemp=size(volOriginal);
   
  if (strcmp(arg, 'axial'))

      volReoriented=volOriginal;      
      
  elseif (strcmp(arg, 'coronal'))
      volReoriented=zeros(stemp(1),stemp(3),stemp(2));
      
      for i=1:stemp(2)
          volReoriented(:,:,i)=imrotate(squeeze(volOriginal(:,i,:)),180);
      end
     
      tt=res(2);
      res(2)=res(3);
      res(3)=tt;
      
      tt=origin(2);
      origin(2)=origin(3);
      origin(3)=tt;
            
  elseif (strcmp(arg, 'sagittal'))
      
    %Create new matrix to store fixed acquisition in axial orientation
    volReoriented=zeros(stemp(3),stemp(1),stemp(2));    

    % Copy axials images in the new matrix
    for i=1:stemp(2)
         volReoriented(:,:,i)=imrotate(squeeze(volOriginal(:,i,:)), 90);
    end
               
    tt1=res(1);
    tt2=res(2);    
    res(1)=res(3);
    res(2)=tt1;
    res(3)=tt2;
    
    tt1=origin(1);
    tt2=origin(2);
    origin(1)=origin(3);
    origin(2)=tt1;
    origin(3)=tt2;
            
    
  end
  
  [path,name,ext]=fileparts(file);
   
  fname=[name,ext];
  
  V.dim=size(volReoriented);
  V.fname=[pathSave,'\ax_',fname];
  
  V.mat(1,1)=-res(1);%-res(1);
  V.mat(2,2)=res(2);
  V.mat(3,3)=res(3);
  
  V.mat(1,4)=abs(origin(1));
  V.mat(2,4)=-abs(origin(2));
  V.mat(3,4)=-abs(origin(3));
        
  spm_write_vol(V,volReoriented);
  
  completePath=V.fname;
  
  
    
  
  
  
  
  